# Comments starts with the symbol '#' and continues to the end of the line.
# ---------------------------------------------------------------------------
# PURPOSE
# Small tutorial demonstrating the basics of MIPS assembly. The tutorial introduces the following MIPS instructions.
#   Instruction		Meaning
#   ----------------------------------------
#   li 			Load Immediate
#   la			Load Address
#   lw			Load Word
#   add			Add
#   addi 	        Add Immediate
#   sw			Store Word
#   move	        Move (copy) register
#
# The tutorial introduces the following SPIM/MARS built in system calls.
#   System call		Code
#   ------------------------
#   print_string	4
#   print_int		1
#   print_char		11
#   exit		10
#
# The tutorial introduces the following MIPS assembler directives.
#   Directive	Description
#   ----------------------------------------------------------------------
#   .data	Subsequent items are stored in the data segment.
#   .text	Subsequent items are put in the user text segment.
#   .space	Allocate n bytes of space in the current segment.
#   .word n	Store the integer n as 4 byte word in the current segment.
#   .asciiz	Store the string str in memory and null-terminate it.
#
# AUTHOR Karl Marklund <karl.marklund@it.uu.se> 2015-12-10, First version.
# modified by <Henrik.Schulze.0353@student.uu.se> 2017-04-04.
# ---------------------------------------------------------------------------

# DATA SEGMENT
# Data used by the program is declared in the data segment.
# ---------------------------------------------------------------------------
	.data
x:	.word 9		# Store value 9 as a 4 byte (a word) size signed integer at label x.
y: 	.space 4	# Reserve four bytes (a word) at label y.
z: 	.space 4	# Reserve four bytes (a word) at label z.
msg:	.asciiz "Hello world!\n"	# Store a null-terminated string at label msg.
nl: 	.asciiz "\n"	# Store new line as a null-terminated string at label nl.

# MAIN
# Text segment (code) for the program.
#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	.text
main:	# Execution starts at the label main.
li $t0, 5	# li - Load Immediate loads a value into a register, pseudo-instruction for addiu $t0, $zero, 5.
# NOTE: li is a pseudo-instruction that gets translated to a addiu (Add Immediate Unsigned) by the assemblator.
la $t1, x	# la - Load Address loads an address into a register: lui $at,x  &  ori $t1,$at,0.  ($1 = $at)
# NOTE: la is a pseudo-instruction that the assemblator translates into two instructions:
# - lui (Load Upper Immediate) sets $ta (dedicated register for translating an address).
# - ori (OR Immediate) loads the memory address of the label into register $t1.
lw $t2, 0($t1)	# lw - Load Word loads a word (= 4 bytes) from address 0($t1) into $t2.
# Above: the value stored at the address 0 + $t1 is loaded into register $t2.
lw $t3, x	# Load the value of the address at label x into $t3; lui $at,x  &  lw $t3,0($at).
# NOTE: when using the above adressing mode (label) the assemblator translates the single lw instruction into two instructions:
# - lui (Load Upper Immediate) stores the memory address of the label into register $at (the address translation register).
# - lw instruction with address mode 0($at), which uses the sum of the immediate 0 and the register $at as the address.
add $t4,$t0,$t2	# add stores the sum of two registers: $t4 := $t0 + $t2, 5 + 9 = 14 => $t4 := 0x0000000e.
addi $t5,$t4,1	# addi - ADD Immediate adds a value: $t5 := $t4 + 1, 14 + 1 = 15 => $t5 := 0x0000000f.
sw $t5, 0($t1)	# sw - Store Word writes a register value to memory. Here $t5 is stored at address 0 + $t1.
sw $t5, z	# Store the value of $t5 to the address at label z: lui $at,x  &  sw $t5,Offs($at); Offs = z - x
# NOTE: when using the above adressing mode (label) the assemblator translates sw to lui + sw like for lw previously.
move $t6, $t5	# move copies the value in a register to another: addu $t6, $zero, $t5. addu = ADD Unsigned.

li $v0, 4	# Set the system call code to 4 (for printing a string) into $v0:  addiu $v0, $zero, 4.
la $a0, msg	# Load into $a0 the address of the label msg: lui $at,x  &  lw $a0, Offset($1); Offset = msg - x
syscall		# Use syscall to perform the instruction with code in $v0 - print the string that $a0 points to.

li $v0, 1	# Set the system call code in $v0 to 1, which instructs to print an integer.
li $a0, 127	# Load value to print into register $a0, pseudo-instruction: addiu $a0, 127 = 0x0000007f.
syscall		# Use the syscall instruction to print the integer. The code 1 (print_int) is in $v0.

li $v0, 4 	# System call code 4 (print_string).
la $a0, nl	# Address of the string to print, label nl : lui $at,x  &  lw $a0, Offset($1); Offset = nl - x
syscall		# Print new line.

li $v0, 1 	# System call code 1 (print_int).
li $a0, 101	# Load value to print into register $a0, pseudo-instruction: addiu $a0, 101 = 0x00000065.
syscall		# Use the syscall instruction to print the prime number.

li $v0, 4 	# System call code 4 (print_string).
la $a0, msg	# Address of the 14 byte string "Hello world!\n".
addi $a0,$a0,12	# Address of the "\n" part of the "Hello world!\n" string.
syscall		# Print (1) new line.

li $v0, 1 	# System call code 1 (print_int).
li $a0, 113	# Load value to print into register $a0, pseudo-instruction: addiu $a0, 113 = 0x00000071.
syscall		# Use the syscall instruction to print the prime number.

li $v0, 1	# System call code 1 (print_int).
move $a0, $t6	# To print whatever content is in $t6.
syscall		# Print $t6 contents.

li $v0, 11 	# System call code 11 (print_char).
li $a0, 0x40	# Load into $a0 the ASCII value of the character to print.
syscall		# Print '@' (ASCII 0x40)

li $v0, 10	# Use system call code 10 (exit) to terminate normally.
syscall		# exit
